import math
from typing import List, Optional, Tuple

from matplotlib import pyplot as plt
from matplotlib.axes import Axes
from matplotlib.figure import Figure

from .style.apply import apply_basic_style


def calc_grid(number_of_subplots: int) -> int:
    return math.ceil(math.sqrt(number_of_subplots))


def make_graph(
    number_of_subplots: int = 1, row: Optional[int] = None, column: Optional[int] = None
) -> Tuple[Figure, List[Axes]]:
    fig = plt.figure()
    grid = calc_grid(number_of_subplots)
    row = row or grid
    column = column or grid
    fig.set_size_inches(8 * column, 8 * row)
    axes = [fig.add_subplot(row, column, i + 1) for i in range(number_of_subplots)]
    apply_basic_style()
    return fig, axes



from .axes.config import AxConfig as AxConfig
from .axes.config import LegendLoc as LegendLoc
from .axes.config import Scale as Scale
from .axes.config import SpineConfig as SpineConfig
from .axes.resolver import apply_ax_config as apply_ax_config
from .axes.resolver import apply_x_config as apply_x_config
from .axes.resolver import apply_y_config as apply_y_config
from .graph import make_graph as make_graph



from enum import Enum
from typing import Optional, Tuple

from pydantic import BaseModel


class Scale(str, Enum):
    LINER = "linear"
    LOG = "log"


class LegendLoc(str, Enum):
    BEST = "best"
    UPPER_LEFT = "upper left"
    UPPER_RIGHT = "upper right"
    LOWER_LEFT = "lower left"
    LOWER_RIGHT = "lower right"
    CENTER_LEFT = "center left"
    CENTER_RIGHT = "center right"
    LOWER_CENTER = "lower center"
    UPPER_CENTER = "upper center"
    CENTER = "center"


class SpineConfig(BaseModel):
    label: str = ""
    scale: Scale = Scale.LINER
    lim: Optional[Tuple[float, float]] = None
    step: Optional[float] = None
    pow: int = 0
    visible: bool = True
    invert: bool = False


class AxConfig(BaseModel):
    x: SpineConfig = SpineConfig()
    y: SpineConfig = SpineConfig()
    is_visible_legend: bool = True
    legends_loc: LegendLoc = LegendLoc.UPPER_LEFT
    bbox_to_anchor: Tuple[float, float] = (1.0, 1.0)
